// slideshow.cpp
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 
//          by:

#include "constants.h"

#include <string>
using std::string;

#include <qpushbutton.h>
#include <qlabel.h>
#include <qgrid.h>
#include <qcolor.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qtextstream.h>


#include "slideshow.h"

#include<qimage.h>

SlideShow::SlideShow( QWidget *parent, const char *name )
        : QVBox( parent, name )
{
	labelCoin = new QPushButton(this);
	labelCoin->setFlat(true);
	labelCoin->setGeometry( 0,0,max_size.width(),max_size.height() ); 	
	labelCoinName = new QLabel(this);
	labelCoinName->setAlignment( AlignHCenter | AlignVCenter );
    labelCoinName->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	QHBox * buttonsCoin = new QHBox(this);
	QPushButton *previousCoin = new QPushButton("previous coin",buttonsCoin, "previous coin");
	QPushButton *nextCoin =     new QPushButton("next coin",buttonsCoin, "next coin");

	QHBox * buttonsCountry = new QHBox(this);
	QPushButton *previousCountry = new QPushButton("previous country",buttonsCountry, "previous country");
	QPushButton *nextCountry =     new QPushButton("next country",buttonsCountry, "next country");

	previousCoin->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
	nextCoin->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
	previousCountry->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
	nextCountry->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
	
	labelCountryName = new QLabel(this);
	labelCountryName->setAlignment( AlignHCenter | AlignVCenter );
    labelCountryName->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	labelCountry = new QPushButton(this);
	labelCountry->setFlat(true);
	labelCountry->setGeometry( 0,0,max_size.width(),max_size.height() ); 	

	connect( nextCoin, SIGNAL(clicked()),		this, SLOT(nextCoin()) );
	connect( previousCoin, SIGNAL(clicked()),	this, SLOT(previousCoin()) );
	connect( nextCountry, SIGNAL(clicked()),	this, SLOT(nextCountry()) );
	connect( previousCountry, SIGNAL(clicked()),this, SLOT(previousCountry()) );

	connect( nextCoin, SIGNAL(clicked()),		this, SLOT(buttonNextCoinPressed()) );
	connect( previousCoin, SIGNAL(clicked()),	this, SLOT(buttonPreviousCoinPressed()) );
	connect( nextCountry, SIGNAL(clicked()),	this, SLOT(buttonNextCountryPressed()) );
	connect( previousCountry, SIGNAL(clicked()),this, SLOT(buttonPreviousCountryPressed()) );

	connect( this, SIGNAL(CountryChanged()), this, SLOT(changeCountrySide()) );
	connect( this, SIGNAL(CoinChanged()), this, SLOT(changeCoinSide()) );
	connect( labelCoin, SIGNAL(clicked()), this, SLOT(CoinPicturePressed()) );
	connect( labelCountry, SIGNAL(clicked()), this, SLOT(CountryPicturePressed()) );

	country=1; coin=0;
}

uint SlideShow::actualCoinNumber(){
	return coin + NumCoins * (country-1);
}

void SlideShow::switchToCoinNumber(uint CoinNumber){
	uint theCoin   = CoinNumber % NumCoins;
	uint theCountry= 1 + (uint) ((CoinNumber-theCoin)/(float)NumCoins);
	this->switchToCoin(theCoin, theCountry);
}


void SlideShow::CoinPicturePressed(){ 
	labelCoin->setDown(false);
	emit Event("CoinPictureClicked");
}
void SlideShow::CountryPicturePressed(){ 
	labelCountry->setDown(false);
	emit Event("CountryPictureClicked");
}


#include <math.h>
#include <qcheckbox.h>
#include <qlayout.h>

void SlideShow::loadPictures(QString picfilepath, QString picfilesuffix, bool &dontShowLoadedPicsResults){
    // check if the picture file format is supported:
    QImageIO qiio;
    QStrList suppFormats=qiio.inputFormats();
    char * f; QString ff; QString allSupported=""; bool supported=false;
    for (f=suppFormats.first(); f!=0; f=suppFormats.next()){
		ff=QString(f);
		allSupported+=ff+" ";
		if (ff==picfilesuffix || ff.lower()==picfilesuffix) supported=true;
	}
    if (!supported){
		QString errorText="Sorry, your version of Qt doesn't support '"+picfilesuffix;
          errorText+="' files, only these filetypes are supported:\n"+allSupported;
          errorText+="\nPlease re-configure Qt and build it again (or ask your admin)\n";
          errorText+="See ' $QTDIR/configure -help | more ' for informations how to...\n";
          errorText+="\nCYE will work, but you won't see the coin pictures! What a pity...";
	    QMessageBox::information( this, "Picture format not supported by Qt", errorText);
	}
        
	uint NumFiles=NumCoins*(NumCountries+1);
	QProgressDialog progress( "Loading pictures...", 
							  "Abort not possible", 
							  NumFiles, this, "progress", TRUE );

	progress.show();
   	QSize s; max_size=QSize(0,0);
    uint i,j, loaded=0, errors=0; 
	QPixmap temp; QString filename;
	QValueList<Q_UINT8> missing;

	for (i=0;i<NumCountries+1;i++){
		for (j=0;j<NumCoins;j++){
			progress.setProgress( i*(NumCountries+1)+j );
			qApp->processEvents() ;
			filename=picfilepath+coinnames[j]+countrynames[i]+"."+picfilesuffix;
			if (temp.load(filename)){
				loaded++;
			}
			else {
				errors++;
				//temp=QPixmap();
				missing.append(i*NumCoins+j);
				// paint a circle, text country and text coin (centered)
				// colour it (three colours)
			}
			// if ( progress.wasCancelled() )  break;
			pics[j][i]=temp;
			s=temp.size();
			if (s.width()>max_size.width()) max_size.setWidth(s.width());
			if (s.height()>max_size.height()) max_size.setHeight(s.height());
		}
	}
	QString text; 
	if (!dontShowLoadedPicsResults && (errors || loaded!=NumFiles)) 
	{
		QDialog *PicLoadD=new QDialog(this, "picdialog", true);
		PicLoadD->setCaption("CYE: You could download (more) coin pictures");
		QGridLayout *grid= new QGridLayout(PicLoadD, 2,2,10);

		text="";
		QTextOStream(&text)<<"Of "<<NumFiles<<" total coins: "<<loaded<<" files loaded, "<<errors;
		text+=" pictures could not be read.";
		text+="\nTo have a nicer program, you might want to check the website for more pictures?";
		QLabel *textL= new QLabel (text, PicLoadD) ;
		QCheckBox *dontShowAgain=new QCheckBox("Don't show this again", PicLoadD);
		QPushButton *okB=new QPushButton("OK", PicLoadD);
		connect(okB, SIGNAL(clicked()), PicLoadD, SLOT(accept()) );
		okB->setDefault(true);
		grid->addMultiCellWidget(textL, 0,0,0,1);
		grid->addWidget(dontShowAgain, 1,0);
		grid->addWidget(okB, 1,1);
		PicLoadD->exec();		
		dontShowLoadedPicsResults=dontShowAgain->isChecked();
	}
	//if (debugging){
	//	text="";
	//	QTextOStream(&text) <<"width="<<max_size.width()<<"  height="<<max_size.height()<<"   ";
	//	QMessageBox::information( this, "CYE: Biggest picture", text );
	//}

	// if (loaded==0)
		max_size=QSize(170,170);
	QValueList<Q_UINT8>::Iterator mC=missing.begin();
	uint coin, country;
	for (; mC!=missing.end(); mC++){
		coin = (*mC) % NumCoins;
		country = (uint) floor ( (*mC) / NumCoins );
		DrawCoin(&(pics[coin][country]), countrynamesWords[country], coinnames[coin], 
			RGB2Color(coincolorInner[coin]), RGB2Color(coincolorOuter[coin]), max_size, 
			coinSizeInner[coin], coinSizeOuter[coin]);
	}

	changeCoinSide(); // to show them
}

void SlideShow::setPics(QPixmap thePics[NumCoins][NumCountries+1], QSize theMaxSize ){
	for (uint i=0; i<NumCoins; i++){
		for (uint j=0; j<NumCountries+1; j++){
			pics[i][j]=thePics[i][j];
		}
	}
	max_size=theMaxSize;
}

QColor SlideShow::RGB2Color(const unsigned char rgb[3]){
	return QColor(rgb[0], rgb[1], rgb[2]);
}

#include <qpainter.h>
#include <qfontmetrics.h>

void SlideShow::DrawCoin(QPixmap *pix, QString countryname, QString coinname, 
						 QColor color1, QColor color2, QSize size, int radius1, int radius2)
{
	QPainter p;
	*pix = QPixmap(size);
	pix->fill( QColor(255,255,255) );
    p.begin( pix );

	p.setPen(NoPen);

	if (radius2!=radius1){
		QBrush  brush2( color2 ); 
		p.setBrush( brush2 );  
		p.drawEllipse( (size.width()-radius2)/2, (size.height()-radius2)/2, radius2, radius2);
	}
	QBrush  brush1( color1 ); 
	p.setBrush( brush1 );  
    p.drawEllipse( (size.width()-radius1)/2, (size.height()-radius1)/2, radius1, radius1);     

	p.setPen(SolidLine);
    QFont font(this->font());
    QFontMetrics fm(font);
	int height=fm.height();
	int width=fm.width((QString)coinname);
	p.drawText((size.width()-width)/2, size.height()/2-height,coinname);
	width=fm.width((QString)countryname);
	p.drawText((size.width()-width)/2, size.height()/2+height,countryname);
    p.end();
}

QPixmap SlideShow::centeredCoin(int coinnumber, int countrynumber){
	QPixmap temp(max_size), source;
	temp.fill( QColor(255,255,255) );
	source= pics[coinnumber][countrynumber];
	bitBlt(&temp, 
		(max_size.width()-source.width())/2, (max_size.height()-source.height())/2,
		& source,
		0,0,source.width(), source.height());
	return temp;
}

// for user observation
void SlideShow::buttonNextCoinPressed(){emit Event("NextCoin");}
void SlideShow::buttonPreviousCoinPressed(){emit Event("PreviousCoin");}
void SlideShow::buttonNextCountryPressed(){emit Event("NextCountry");}
void SlideShow::buttonPreviousCountryPressed(){emit Event("PreviousCountry");}


void SlideShow::changeCountrySide()
{
	labelCountry->setPixmap(centeredCoin(coin,country));
	labelCountryName->setText(countrynamesWords[country]);
	// emit focusChanged(coin, country);
}

void SlideShow::nextCountry(){
	country++;	if (country>=NumCountries+1) country=1;
	//emit CountryChanged();
	changeCountrySide();
	emit focusChanged(coin, country);
}

void SlideShow::previousCountry(){
	country--; if (country<1) country=NumCountries;
	// emit CountryChanged();
	changeCountrySide();
	emit focusChanged(coin, country);
}


void SlideShow::changeCoinSide()
{
	labelCoin->setPixmap(centeredCoin(coin,0));
	labelCoinName->setText(coinnames[coin]);
	changeCountrySide();
	// emit focusChanged(coin, country);
}

void SlideShow::nextCoin(){
	coin++;	
	if (coin>=NumCoins) coin=0;
	// emit CoinChanged();
	changeCoinSide();
	emit focusChanged(coin, country);
}

void SlideShow::previousCoin(){
	if (coin==0) coin=NumCoins-1;
        else coin--;
	// emit CoinChanged();
	changeCoinSide();
	emit focusChanged(coin, country);
}

void SlideShow::switchToCoin(uint toCoin, uint toCountry){
  if ((toCoin!=coin ) && toCoin<NumCoins /* && toCoin>=0 */ ){
		coin=toCoin; 
		changeCoinSide();
		changeCountrySide();
	}
	if (toCountry !=country && toCountry<= NumCountries && toCountry>= 1){
		country=toCountry;
		changeCountrySide();
	}
}


//QSizePolicy SlideShow::sizePolicy() const
//{
//    return QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );
//	//return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
//}
