// usage.h
// usage of the program is observed
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 29.5.2002
//          by: AK

#ifndef USAGE_H
#define USAGE_H

#include "classes.h"

#include <qglobal.h>

class UserEvent;
class Usage;
class ButtonsData;

class UserEvent{
private:
	Q_UINT32 time; // use only 24bits (to store it in the db as MEDIUMINT)
	               // 123.45 sec --> 12345 hundredth-seconds
	Q_UINT16 theID;
public:
	float TimeSecs(){return time/100.0;};
	long  TimeMSecs(){return (long)(time*10.0);};
	Q_UINT16 ID(){return theID;};

	UserEvent (long msec, Q_UINT16 eventID) : time((Q_UINT16)(msec/10.0)), theID(eventID){};
	UserEvent () : time(0), theID(0) {};
	UserEvent (const UserEvent &rhs) : time(rhs.time), theID(rhs.theID) {};

	friend QDataStream& operator<< (QDataStream &s, UserEvent UE);
	friend QDataStream& operator>> (QDataStream &s, UserEvent &UE);
	
	friend bool operator== (UserEvent & , UserEvent &);
};


#include <qdatetime.h>
#include <qtextstream.h>
#include <qobject.h>
#include <qmap.h>
#include <qlistview.h>
#include <qvaluelist.h>

class Usage : public QObject {
	Q_OBJECT
public:
	QValueList<UserEvent>		Session;
	QMap<Q_UINT16, Q_UINT32>	TotalEvents;
	bool StoreEvent(QString eventName);

	Usage(QWidget * parent=0, const char * name=0);
	void startObservation(const QMap<Q_UINT16, Q_UINT32> &OldTotalEvents);

	void setDataTo(QValueList<UserEvent>& oneSession, 
		             QMap<Q_UINT16, Q_UINT32>& oneTotalEvents);
	void setDataTo(QMap<Q_UINT16, Q_UINT32>& oneTotalEvents);
	void setDataTo(QValueList<UserEvent>& oneSession);

	void PrintSessionTo(QTextOStream &s);
	void PrintTotalEventsTo(QTextOStream &s);
	void PrepareSessionListView(QListView * LV);
	void UpdateSessionListView(QListView * LV);
	void PrepareAllEventsListView(QListView * LV);
	void UpdateAllEventsListView(QListView * LV);

	Q_UINT32 AllEventsCreateListViewItemChildrenOf(QListViewItem * LV);
	         // returns total number of events
	Q_UINT32 SessionEventsCreateListViewItemChildrenOf(QListViewItem * LV);
		     // returns total number of events

	static char* giveAllEventsAsTextTupels(char* thirdcolumn);

public slots:
	void toggleObservationAllowed(bool status);

private slots:
	void restartTimer();

signals:
	void Event(QString eventname);

private:
	bool					ObservationAllowed;
	QTime					timer;
	QMap<QString, Q_UINT16>	mapEventName2ID;
	QMap<Q_UINT16, QString>	mapID2EventName;
	const QMap<QString, Q_UINT16> & EventName2ID;
	const QMap<Q_UINT16, QString> & ID2EventName;

	void initEventIDTextMapping();
	void PrintEventTo(QTextOStream & s, QValueList<UserEvent>::Iterator UE);

	void testMapping();
	void testSession();
};


#endif // USAGE_H
